__version__ = "1.1"
__package_name__ = "doubo"
__license__ = "MIT"
__description__ = "Place a buy and a sell order in one go on MetaTrader5"
__author__ = "yh2fire"
__author_email__ = "zykxing@163.com"
